@@ -934,13 +934,12 @@ copy_button_cap(const struct tablet_dispatch *tablet,
 		set_bit(tool->buttons, button);
 }
 
+#if HAVE_LIBWACOM
 static inline int
 tool_set_bits_from_libwacom(const struct tablet_dispatch *tablet,
 			    struct libinput_tablet_tool *tool)
 {
 	int rc = 1;
-
-#if HAVE_LIBWACOM
 	WacomDeviceDatabase *db;
 	const WacomStylus *s = NULL;
 	int code;
@@ -997,9 +996,10 @@ tool_set_bits_from_libwacom(const struct tablet_dispatch *tablet,
 		copy_axis_cap(tablet, tool, LIBINPUT_TABLET_TOOL_AXIS_PRESSURE);
 
 	rc = 0;
-#endif
+
 	return rc;
 }
+#endif
 
 static void
 tool_set_bits(const struct tablet_dispatch *tablet,
@@ -1506,7 +1506,7 @@ tablet_calculate_arbitration_rect(struct tablet_dispatch *tablet)
 	 * If the stylus is tilted left (tip further right than the eraser
 	 * end) assume left-handed mode.
 	 *
-	 * Obviously if we'd run out of the boundaries, we rescale the rect
+	 * Obviously if we'd run out of the boundaries, we clip the rect
 	 * accordingly.
 	 */
 	if (tablet->axes.tilt.x > 0) {
@@ -1519,14 +1519,14 @@ tablet_calculate_arbitration_rect(struct tablet_dispatch *tablet)
 	}
 
 	if (r.x < 0) {
-		r.w -= r.x;
+		r.w += r.x;
 		r.x = 0;
 	}
 
 	r.y = mm.y - 100;
 	r.h = 250;
 	if (r.y < 0) {
-		r.h -= r.y;
+		r.h += r.y;
 		r.y = 0;
 	}
 
