@@ -59,6 +59,9 @@ enum configuration_options {
 	OPT_PROFILE,
 	OPT_DISABLE_SENDEVENTS,
 	OPT_APPLY_TO,
+	OPT_CUSTOM_POINTS,
+	OPT_CUSTOM_STEP,
+	OPT_CUSTOM_TYPE,
 };
 
 #define CONFIGURATION_OPTIONS \
@@ -87,7 +90,10 @@ enum configuration_options {
 	{ "set-profile",               required_argument, 0, OPT_PROFILE }, \
 	{ "set-tap-map",               required_argument, 0, OPT_TAP_MAP }, \
 	{ "set-speed",                 required_argument, 0, OPT_SPEED },\
-	{ "apply-to",                  required_argument, 0, OPT_APPLY_TO }
+	{ "apply-to",                  required_argument, 0, OPT_APPLY_TO },\
+	{ "set-custom-points",         required_argument, 0, OPT_CUSTOM_POINTS },\
+	{ "set-custom-step",           required_argument, 0, OPT_CUSTOM_STEP },\
+	{ "set-custom-type",           required_argument, 0, OPT_CUSTOM_TYPE }
 
 enum tools_backend {
 	BACKEND_NONE,
@@ -114,6 +120,10 @@ struct tools_options {
 	int dwtp;
 	enum libinput_config_accel_profile profile;
 	char disable_pattern[64];
+	enum libinput_config_accel_type custom_type;
+	double custom_step;
+	size_t custom_npoints;
+	double *custom_points;
 };
 
 void tools_init_options(struct tools_options *options);
